import React from 'react'

const NotFound = () => (
    <div className="alert alert-danger">
        <strong>Nothing found here</strong> &ndash; try the main menu.
    </div>
)

export default NotFound
