"""Some Metadata for model parameters, understood as single values with a
default and an expected range.

"""
from smif.parameters.narrative import Narrative
from smif.parameters.parameters import Parameter, ParameterList

__all__ = ['Parameter', 'ParameterList', 'Narrative']
